;V1.1
;Use this include for Windows XP
;Note:::Requires SP2 or greater
#include-Once
#include<String.au3>
; ===============================================================================================================================
; Name...........: _PowerScheme_List
; Description ...: Returns a 2d array of power schemes on the computer
; Syntax.........: _PowerScheme_List()
; Parameters ....: None
; Return values .: Success - A 2D Array in the follwoing format
;                              $Array[0][0]=Number of items
;                              $Array[X][0]=Hex string of power scheme
;                              $Array[X][1]=Name of power scheme
;                  Failure - an empty string ("")
;                  |@Error  - 1 = Failed to execute PowerCFG
;                  |@Error  - 2 = Data from PowerCFG.exe badly formatted
;                  |@Error  - 3 = OS is not Windows XP
; Author ........: Michael Michta
; ===============================================================================================================================
Func _PowerScheme_List()
	If @OSVersion<>"Win_XP" then Return SetError(3,0,"")
	Local $hPID = Run("PowerCFG /L", @SystemDir, @SW_HIDE, 0x2)
	If @error Then Return SetError(1, 0, "")
	ProcessWaitClose($hPID)
	Local $asLines = StringSplit(StringTrimRight(StringTrimLeft(StdoutRead($hPID), 50), 2), @CRLF, 1)
	Local $asArray[$asLines[0] + 1][2]
	$asArray[0][0] = $asLines[0]
	For $iCounter = 1 To $asLines[0]
		$asArray[$iCounter][0] = StringStripWS(StringToBinary($asLines[$iCounter]),3)
		$asArray[$iCounter][1] = StringStripWS($asLines[$iCounter],3)
	Next
	Return $asArray
EndFunc   ;==>_PowerScheme_List
; ===============================================================================================================================
; Name...........: _PowerScheme_GetActive
; Description ...: Returns the active power scheme
; Syntax.........: _PowerScheme_GetActive()
; Parameters ....: None
; Return values .: Success - an Array in the follwoing format
;                              $Array[0][0]= Hex string of power scheme
;                              $Array[X][1]= Name of power scheme
;                  Failure - an empty string ("")
;                  |@Error  - 1 = Failed to execute PowerCFG
;                  |@Error  - 2 = Data from PowerCFG.exe badly formatted
;                  |@Error  - 3 = OS is not Windows XP
; Author ........: Michael Michta
; ===============================================================================================================================
Func _PowerScheme_GetActive()
	If @OSVersion<>"Win_XP" then Return SetError(3,0,"")
	Local $hPID = Run("PowerCFG /Q", @SystemDir, @SW_HIDE, 0x2)
	If @error Then Return SetError(1, 0, "")
	ProcessWaitClose($hPID)
	Local $sRead = StdoutRead($hPID)
	Local $asArray[2]
	Local $asBetween = _StringBetween($sRead, "Name                       ", "Numerical ID")
	If Not IsArray($asBetween) Then Return SetError(2, 0, "")
	$asBetween[0]=StringTrimRight($asBetween[0],2)
	$asArray[0] = StringStripWS(StringToBinary($asBetween[0]),3)
	$asArray[1] = StringStripWS($asBetween[0],3)
	Return $asArray
EndFunc   ;==>_PowerScheme_GetActive
; ===============================================================================================================================
; Name...........: _PowerScheme_SetActive
; Description ...: Sets the active power scheme
; Syntax.........: _PowerScheme_SetActive($sHex)
; Parameters ....: $sHex - The hex string of the power scheme to set as active
; Return values .: Success - 1
;                  Failure - 0
;                  |@Error  - 1 = OS is not Windows XP
; Author ........: Michael Michta
; ===============================================================================================================================
Func _PowerScheme_SetActive($sHex)
	If @OSVersion<>"Win_XP" then Return SetError(1,0,0)
	Local $iRet = RunWait('PowerCFG /SETACTIVE "' & BinaryToString($sGUID) &'"', @SystemDir, @SW_HIDE, 0x2)
	Return Not $iRet
EndFunc   ;==>_PowerScheme_SetActive
