;V1.1
;Use this include for Windows Vista/7
#include-Once
#include<String.au3>
; ===============================================================================================================================
; Name...........: _PowerScheme_List
; Description ...: Returns a 2d array of power schemes on the computer
; Syntax.........: _PowerScheme_List()
; Parameters ....: None
; Return values .: Success - A 2D Array in the following format
;                              $Array[0][0]=Number of items
;                              $Array[X][0]=GUID of power scheme
;                              $Array[X][1]=Name of power scheme
;                  Failure - an empty string ("")
;                  |@Error  - 1 = Failed to execute PowerCFG
;                  |@Error  - 2 = Data from PowerCFG.exe badly formatted
;                  |@Error  - 3 = OS is not Vista or Windows 7
; Author ........: Michael Michta
; ===============================================================================================================================
Func _PowerScheme_List()
	If Not (@OSVersion="Win_7" or @OSVersion="Win_Vista") then Return SetError(3,0,"")
	Local $hPID = Run("PowerCFG -L", @SystemDir, @SW_HIDE, 0x2)
	If @error Then Return SetError(1, 0, "")
	ProcessWaitClose($hPID)
	Local $asLines = StringSplit(StringTrimRight(StringTrimLeft(StdoutRead($hPID), 74), 2), @CRLF, 1)
	Local $asArray[$asLines[0] + 1][2]
	$asArray[0][0] = $asLines[0]
	For $iCounter = 1 To $asLines[0]
		If StringLeft($asLines[$iCounter], 18) <> "Power Scheme GUID:" Then ContinueLoop
		Local $sLine = StringTrimLeft($asLines[$iCounter], 19)
		$asArray[$iCounter][0] = StringStripWS(StringLeft($sLine, 36),3)
		Local $asBetween = _StringBetween($sLine, "(", ")")
		If Not IsArray($asBetween) Then Return SetError(2, 0, "")
		$asArray[$iCounter][1] = StringStripWS($asBetween[0],3)
	Next
	Return $asArray
EndFunc   ;==>_PowerScheme_List

; ===============================================================================================================================
; Name...........: _PowerScheme_GetActive
; Description ...: Returns the active power scheme
; Syntax.........: _PowerScheme_GetActive()
; Parameters ....: None
; Return values .: Success - an Array in the following format
;                              $Array[0][0]= GUID of power scheme
;                              $Array[X][1]= Name of power scheme
;                  Failure - an empty string ("")
;                  |@Error  - 1 = Failed to execute PowerCFG
;                  |@Error  - 2 = Data from PowerCFG.exe badly formatted
;                  |@Error  - 3 = OS is not Vista or Windows 7
; Author ........: Michael Michta
; ===============================================================================================================================
Func _PowerScheme_GetActive()
	If Not (@OSVersion="Win_7" or @OSVersion="Win_Vista") then Return SetError(3,0,"")
	Local $hPID = Run("PowerCFG -GETACTIVESCHEME", @SystemDir, @SW_HIDE, 0x2)
	If @error Then Return SetError(1, 0, "")
	ProcessWaitClose($hPID)
	Local $sRead = StringTrimLeft(StdoutRead($hPID), 18)
	Local $asArray[2]
	$asArray[0] = StringStripWS(StringLeft($sRead, 37),3)
	Local $asBetween = _StringBetween($sRead, "(", ")")
	If Not IsArray($asBetween) Then Return SetError(2, 0, "")
	$asArray[1] = StringStripWS($asBetween[0],3)
	Return $asArray
EndFunc   ;==>_PowerScheme_GetActive
; ===============================================================================================================================
; Name...........: _PowerScheme_SetActive
; Description ...: Sets the active power scheme
; Syntax.........: _PowerScheme_SetActive($sGUID)
; Parameters ....: $sGUID - The GUID of the power scheme to set as active
; Return values .: Success - 1
;                  Failure - 0
;                  |@Error  - 1 = OS is not Vista or Windows 7
; Author ........: Michael Michta
; ===============================================================================================================================
Func _PowerScheme_SetActive($sGUID)
	If Not (@OSVersion="Win_7" or @OSVersion="Win_Vista") then Return SetError(1,0,0)
	Local $iRet = RunWait("PowerCFG -SETACTIVE " & $sGUID, @SystemDir, @SW_HIDE, 0x2)
	Return Not $iRet
EndFunc   ;==>_PowerScheme_SetActive
